function initSPIInterface() {
  function PMICSPI(packetParser, sendPacket) {
    OCFBaseEx.apply(this, arguments);
    this.handlers = [
      reply_SPI_Enable,
      reply_SPI_Config,
      reply_SPI_WriteRegister,
      reply_SPI_ReadRegister
    ];
  }
  PMICSPI.prototype = Object.create(OCFBaseEx.prototype);
  PMICSPI.prototype.constructor = PMICSPI;
  OCFBaseEx.prototype.add_interface("pmicspi", PMICSPI);

  /**
   * Initialize the symbols for device, and it should invoked by GC framework only.
   * See its base method for description
   */
  PMICSPI.prototype.initSymbolsForDevice = function(settings, registerModel) {
    OCFBaseEx.prototype.initSymbolsForDevice.apply(this, arguments);
    // registerModel.addUserPreference('$cs_gpio', settings.cs_gpio); // should not be user preference that overwrites App intent.
  };

  // system.json properties.
  // {
  //     "name": a string, "spi" in this case
  //   , "unit": a number, inteface unit
  //   , "cs_gpio": a string, chip select gpio pin, e.g. 'PN2'; or a number as the pin mask of the gpio pin
  //   , "dataEndian": 'big', or 'little' (default), data endianess
  //   ,  "read_write_register_protocol": {
  //          "write_cmd": hex string or number, write command
  //        , "read_cmd": hex string or number, read command
  //        , "addr_bitshift": a number, address bit shift
  //        , "data_bitshift": a number, data bit shift
  //        , "parity": "odd" or "even"
  //        , "parity_bitshift": a number, parity bit shift
  //        , "payload_endian": 'big' or 'little', payload endianess
  //        , "payload_bitsize": a number, payload bit size
  //        , "reply_payload_data_start": a number, payload start index for register data, see JS array slice for pos/neg number
  //        , "reply_payload_data_end": a number, payload end index for register data, see JS array slice for pos/neg number
  //        , "dataEndian": 'big', or 'little' (default), data endianess
  //      }
  //    , "config" : [
  //        {
  //          "command": "enable" or "config"
  //          ,"unit": (Optiona) unit number for the command
  //
  //          ,"enable": 'enable' command - true (enable) or false (disable)
  //
  //          ,"bitrate" : 'config' command - bit rate
  //          ,"protocol" : 'config' command - a number indicating ssi protocol
  //          ,"datawidth" : 'config' command - data width
  //          ,"cs_mode" : 'config' command - SPI active mode, 1 active high; 0 active low
  //          ,"crc_en" : 'config' command - true or false for crc
  //          ,"cs_change" : 'config' command - cs_mode change between SPI word. 0: no change; 1: change
  //        }
  //        , ... (etc.)
  //      ]
  // }

  /**
   * Initialize the interface instance, and it should invoked by GC framework only.
   * See its base method for description
   */
  PMICSPI.prototype.init = function(info) {
    this.cs_gpio = info.cs_gpio;
    let self = this;
    if (info.read_write_register_protocol) {
      info.read_write_register_protocol.write_cmd =
        parseInt(info.read_write_register_protocol.write_cmd) || 0;
      info.read_write_register_protocol.read_cmd =
        parseInt(info.read_write_register_protocol.read_cmd) || 0;
    }
    // should not do this as a binding nor user preferece nor prop binding nor pseudo register
    // if (info.cs_gpio_hdl === undefined) {
    //    info.cs_gpio_hdl = function(oldValue, newValue) {
    //        self.cs_gpio = newValue;
    //    }
    //    info._registerModel.getBinding('$cs_gpio').addChangedListener({onValueChanged: info.cs_gpio_hdl});
    // }
    return OCFBaseEx.prototype.init.apply(this, arguments);
  };

  PMICSPI.prototype.getPageNumberforRegister = function(info) {
    let deviceAddrs;
    // we use "deviceAddrs" to retrieve page based on the system.json setup
    if (info.parentGroup) {
      deviceAddrs = info.parentGroup.deviceAddrs;
    } else {
      deviceAddrs = info.page || "page0";
    }

    const page = deviceAddrs.replace("page", "");
    return Number(page);
  };

  /** See its base method for description */
  PMICSPI.prototype.read = function(regInfo) {
    if (!regInfo._num_read_bytes) {
      let sz = parseInt(regInfo.size); // size in bits
      regInfo._num_read_bytes = (sz + 7) >> 3; // size in bytes
    }
    if (this._custom_cb && this._custom_cb.read) {
      return this._custom_cb.read(regInfo);
    }
    let self = this;
    let protocol =
      regInfo.read_write_register_protocol ||
      this.info.read_write_register_protocol;
    let reg_addr =
      (regInfo.writeAddr || regInfo.addr) << (protocol.addr_bitshift || 0);
    let page = this.getPageNumberforRegister(regInfo);
    let num_read_bytes = regInfo._num_read_bytes;
    let chip_select = regInfo.chipSelect;
    let write_content = [];
    console.log(
        `SPI_READ -> Register Addr: ${reg_addr} page: ${page} ${chip_select}`
        )
    return this.SPI_ReadRegister(
      this.unit,
      reg_addr,
      page,
      write_content,
      num_read_bytes,
      chip_select
    ).then(function(payload) {
      if (payload && protocol) {
        payload = payload.slice(
          protocol.reply_payload_data_start,
          protocol.reply_payload_data_end
        );
      }
      if (payload && payload.length > 1) {
        return payload;
      } else {
        return self.bytes_to_value(payload, protocol && protocol.dataEndian);
      }
    });
  };

  /** See its base method for description */
  PMICSPI.prototype.write = function(regInfo, data) {
    if (this._custom_cb && this._custom_cb.write) {
      return this._custom_cb.write(regInfo, data);
    }
    let protocol =
      regInfo.read_write_register_protocol ||
      this.info.read_write_register_protocol;
    let data_bitshift = protocol.data_bitshift;
    let write_content = [];
    let reg_addr =
      (regInfo.writeAddr || regInfo.addr) << (protocol.addr_bitshift || 0);
    let page = this.getPageNumberforRegister(regInfo);
    if (data.constructor === Array) {
      write_content = data;
    } else {
      if (!regInfo._bit_size) {
        regInfo._bit_size = parseInt(regInfo.size);
      }
      data = data << (data_bitshift || 0);
      this.value_to_bytes(
        data,
        regInfo._bit_size,
        protocol.payload_endian,
        write_content
      );
    }
    let payload = this.flatten(write_content);
    let chip_select = regInfo.chipSelect;

    console.log(
        `SPI_WRITE -> Register Addr: ${reg_addr} page: ${page} data: ${payload} ${chip_select}`
        )
    return this.SPI_WriteRegister(this.unit, reg_addr, page, payload, chip_select);
  };

  /**
   * Configure the inteface, and it should invoked by GC framework only.
   */
  PMICSPI.prototype.ensureConfigured = function(config_seq) {
    let promises = [];
    let seq_len = config_seq ? config_seq.length : 0;
    for (let i = 0; i < seq_len; i++) {
      let config = config_seq[i];
      switch (config.command) {
        case "enable":
          promises.push(
            this.SPI_Enable(config.unit || this.unit, config.enable)
          );
          break;
        case "config":
          promises.push(
            this.SPI_Config(
              config.unit || this.unit,
              config.bitrate,
              config.protocol,
              config.datawidth,
              config.cs_mode,
              config.crc_en,
              config.cs_change
            )
          );
          break;
      }
    }
    return Q.all(promises);
  };

  // Reference ocf_common.h
  // #define PMICSPI_Interface       0x1B
  // typedef enum
  // {
  //    ocCmd_PMICSPI_Enable = 0x00,
  //    ocCmd_PMICSPI_Config,
  //    ocCmd_PMICSPI_WriteRegister,
  //    ocCmd_PMICSPI_ReadRegister,
  // } SPI_CMD;
  let PMICSPI_Interface = 0x1b;
  let ocCmd_PMICSPI_Enable = 0x00;
  let ocCmd_PMICSPI_Config = 0x01;
  let ocCmd_PMICSPI_WriteRegister = 0x02;
  let ocCmd_PMICSPI_ReadRegister = 0x03;

  /** Get interface type */
  PMICSPI.prototype.get_interface_type = function() {
    return PMICSPI_Interface;
  };

  /**
   * Enable or Disable the inteface
   * @param {Number} unit interface unit
   * @param {Boolean} enable enable if true, disable if false
   * @return {Promise}
   */
  PMICSPI.prototype.SPI_Enable = function(unit, enable) {
    // uint32_t unit, bool enable
    let params = [this.uint32_leb4(unit), this.uint32_leb4(enable ? 1 : 0)];
    return this.h2c_command(
      PMICSPI_Interface,
      unit,
      ocCmd_PMICSPI_Enable,
      params
    );
  };

  /** Internal function */
  var reply_SPI_Enable = function(self, qdef, unit, status, replypkt) {
    if (status == 0) {
      let enable = self.leb4_uint32(replypkt.params[1]) == 1;
      self.unit_state[unit].current = enable ? 1 : 0;
      qdef && qdef.resolve(enable);
    } else {
      qdef &&
        qdef.reject(
          self.status_msg(
            status,
            replypkt.payload
              ? self.bytes_ascii(replypkt.payload)
              : "Failed in Enable/Disable"
          )
        );
    }
  };

  /**
   * Configure the interface
   * @param {Number} unit interface unit
   * @param {Number} bitrate bit rate
   * @param {Number} protocol
   * @param {Number} datawidth
   * @param {Number} cs_mode
   * @param {Number} crc_en
   * @param {Number} cs_change
   * @return {Promise}
   */
  PMICSPI.prototype.SPI_Config = function(
    unit,
    bitrate,
    protocol,
    datawidth,
    cs_mode,
    crc_en,
    cs_change
  ) {
    // uint32_t unit, uint32_t bit_rate, uint32_t protocol (spi mode), uint32_t datawidth, uint32_t cs_mode, uint32_t cs_change
    // bitrate: in units of 1kHz; max rate is 60000 kHz
    // protocol: see ssi.h
    //  #define SSI_FRF_MOTO_MODE_0  0x00000000 // Moto fmt, polarity 0, phase 0
    //  #define SSI_FRF_MOTO_MODE_1  0x00000002 // Moto fmt, polarity 0, phase 1
    //  #define SSI_FRF_MOTO_MODE_2  0x00000001 // Moto fmt, polarity 1, phase 0
    //  #define SSI_FRF_MOTO_MODE_3  0x00000003 // Moto fmt, polarity 1, phase 1
    //  #define SSI_FRF_TI           0x00000010 // TI frame format
    //  #define SSI_FRF_NMW          0x00000020 // National microwire frame format
    //  #define SSI_ADV_MODE_BI_READ   0x00000140
    //  #define SSI_ADV_MODE_BI_WRITE  0x00000040
    // datawidth: word width between 4 and 16 bits
    // cs_mode: SPI active mode: 1 active high; 0 active low
    // crc_en: 'config' command - true or false for crc
    // cs_change: cs_mode change between SPI word. 0: no change; 1: change
    let params = [
      this.uint32_leb4(unit),
      this.uint32_leb4(bitrate),
      this.uint32_leb4(protocol),
      this.uint32_leb4(datawidth),
      this.uint32_leb4(parseInt(`${+crc_en}${cs_mode}`, 2)),
      this.uint32_leb4(cs_change)
    ];
    return this.h2c_command(
      PMICSPI_Interface,
      unit,
      ocCmd_PMICSPI_Config,
      params
    );
  };

  /** Internal function */
  var reply_SPI_Config = function(self, qdef, unit, status, replypkt) {
    if (status == 0) {
      self.unit_state[unit].current = 2;
      qdef && qdef.resolve(true);
    } else {
      qdef &&
        qdef.reject(
          self.status_msg(
            status,
            replypkt.payload
              ? self.bytes_ascii(replypkt.payload)
              : "Failed in Config"
          )
        );
    }
  };

  /**
   * Write Register
   * This function writes the number of bytes to a specified page and register
   * @param {Number} unit interface unit
   * @param {Number} register_addr First register address to write to
   * @param {Number} page Page to write to
   * @param {Array} pWriteBuffer data to write
   * @return {Promise}
   */
  PMICSPI.prototype.SPI_WriteRegister = function(
    unit,
    register_addr,
    page,
    pWriteBuffer,
    chip_select
  ) {
    let params = [
      this.uint32_leb4(unit),
      this.uint32_leb4(register_addr),
      this.uint32_leb4(page),
      this.uint32_leb4(chip_select)
    ];
    return this.h2c_command(
      PMICSPI_Interface,
      unit,
      ocCmd_PMICSPI_WriteRegister,
      params,
      pWriteBuffer
    );
  };

  /** Internal function */
  var reply_SPI_WriteRegister = function(self, qdef, unit, status, replypkt) {
    if (!qdef) {
      return;
    }
    if (status == 0) {
      qdef.resolve(replypkt.payload);
    } else {
      qdef.reject(
        self.status_msg(
          status,
          replypkt.payload
            ? self.bytes_ascii(replypkt.payload)
            : "Failed in Write Register"
        )
      );
    }
  };

  /**
   * Read Register
   * This function reads the number of bytes from a specified page starting from a given register
   * @param {Number} unit interface unit
   * @param {Number} register_addr First register address to read
   * @param {Number} page Page to write to
   * @param {Array} pWriteBuffer buffer to hold read data
   * @param {Number} no_of_bytes Number of bytes to read (cannot exceed 2048)
   * @return {Promise}
   */
  PMICSPI.prototype.SPI_ReadRegister = function(
    unit,
    register_addr,
    page,
    pWriteBuffer,
    no_of_bytes,
    chip_select
  ) {
    let params = [
      this.uint32_leb4(unit),
      this.uint32_leb4(register_addr),
      this.uint32_leb4(page),
      this.uint32_leb4(no_of_bytes),
      this.uint32_leb4(chip_select)
    ];
    return this.h2c_command(
      PMICSPI_Interface,
      unit,
      ocCmd_PMICSPI_ReadRegister,
      params,
      pWriteBuffer
    );
  };

  /** Internal function */
  var reply_SPI_ReadRegister = function(self, qdef, unit, status, replypkt) {
    if (!qdef) {
      return;
    }
    if (status == 0) {
      qdef.resolve(replypkt.payload);
    } else {
      qdef.reject(
        self.status_msg(
          status,
          replypkt.payload
            ? self.bytes_ascii(replypkt.payload)
            : "Failed in Read Register"
        )
      );
    }
  };
}

if (gc.databind && gc.databind.AEVMCodec) {
  initSPIInterface();
} else {
  document.addEventListener("WebComponentsReady", initSPIInterface);
}
